; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			
; Timer1 (16Bit ) Vorteiler 												
;																			
; TCCR1B ==> ICNC1 ICES1 RESERVIERT WGM13 WGM12 CS12 CS11 CS10 				
;																			
;CS12 CS11 CS10 Description													
;	0	0 	0 	No clock source. (Timer/Counter stopped)					
;	0 	0 	1 	clkI/O/1    (No prescaling)									
;	0 	1 	0 	clkI/O/8    (From prescaler)								
; 	0 	1 	1 	clkI/O/64   (From prescaler)								
; *	1 	0 	0 	clkI/O/256  (From prescaler)								
; 	1 	0 	1 	clkI/O/1024 (From prescaler)								
;	1 	1 	0 	External clock source on T1 pin. Clock on falling edge.		
;	1 	1 	1 	External clock source on T1 pin. Clock on rising edge.		
;																			
TIMER1_INITIALISIERUNG:
	ldi temp,(1<<CS12)|(0<<CS11) |(0<<CS10)
	out TCCR1B,temp          
	; Timer 1 Startwert 	
	ldi temp,HIGH(TIMERWERT1)
	out tcnt1H,temp         
	ldi temp,LOW (TIMERWERT1) 
	out tcnt1L,temp     
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Timer 0+1+2 berlauf-Int aktivieren (Timer/Counter Interrupt Mask)		
;																			
; TIMSK ==> OCIE2 TOIE2 TICIE1 OCIE1A OCIE1B TOIE1 R TOIE0  				
;																			
TIMER_AKTIVIEREN:
	ldi temp,(0<<OCIE2)|(0<<TOIE2)|(0<<TICIE1)|(0<<OCIE1A)|(0<<OCIE1B)|(1<<TOIE1)|(0<<TOIE0)
	out TIMSK,temp   
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; bei 16 MHz ==> 10ms 															
TIM1_OVF:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push TEMP1
	push TEMP2
	push ZL
	push ZH
; ----------------------------------
; Timer Startwert	 				
	ldi temp,HIGH(TIMERWERT1)
	out tcnt1H,temp         
	ldi temp,LOW (TIMERWERT1) 
	out tcnt1L,temp
; ----------------------------------
	rcall QUARZ_ZEIT_10ms
	rcall ZEIT_10ms
	rcall DCF_UHR
; ----------------------------------
	pop ZH
	pop ZL
	pop TEMP2
	pop TEMP1
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 				
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
QUARZ_ZEIT_10ms:
; 100-el Sekunden		
	LDS temp,(adr_QUARZ_ss100)	; laden			
	inc temp					; +1			
	cpi temp,100				; vergleich		
	brsh QUARZ_ZEIT_SS			; gleich/grer	
	STS (adr_QUARZ_ss100),temp	; speichern		
	ret						
; Sekunden				
QUARZ_ZEIT_SS:
	STS(adr_QUARZ_ss100),NULL

	LDS temp,(adr_QUARZ_ss)	
	inc temp
	cpi temp,60
	brsh QUARZ_ZEIT_MM
	STS (adr_QUARZ_ss),temp	
	ret	
; Minuten				
QUARZ_ZEIT_MM:
	STS (adr_QUARZ_ss),NULL	

	LDS temp,(adr_QUARZ_mm)	
	inc temp
	cpi temp,60
	brsh QUARZ_ZEIT_HH
	STS (adr_QUARZ_mm),temp	
	ret	
; Stunden				
QUARZ_ZEIT_HH:
	STS (adr_QUARZ_mm),NULL	

	LDS temp,(adr_QUARZ_hh)	
	inc temp
	cpi temp,24
	brsh QUARZ_ZEIT_TAG
	STS (adr_QUARZ_hh),temp	
	ret
; TAG						
QUARZ_ZEIT_TAG:
	STS (adr_QUARZ_hh),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZEIT_10ms:
; 100-el Sekunden		
	LDS temp,(adr_ss100)	; aus SRAM laden			
	inc temp				; +1						
	cpi temp,100			; vergleich mit 100			
	brsh ZEIT_SS			; ist gleich oder grer	
	STS (adr_ss100),temp	; in SRAM speichern			
	ret						; zurck					
; Sekunden				
ZEIT_SS:
	STS(adr_ss100),NULL
	; PIN-WERT (fr Automatikmodus)
	LDS temp,(adr_DCF_PIN_WERT)
	STS(adr_DCF_PIN_WERT_OLD),temp
	STS(adr_DCF_PIN_WERT),NULL
	; PROGRAMMFREIGABE	
	STS(adr_FREIGABE_PROGRAMM),EINS
	; BETRIEBSSTUNDEN	
	rcall ZEIT_BETRIEBSSTUNDEN
	; DCF PHASE TIMER	
	rcall TIMER_PHASE
	LDS temp,(adr_ss)	; aus SRAM laden	
	inc temp
	cpi temp,60
	brsh ZEIT_MM
	STS (adr_ss),temp	; in SRAM speichern	
	ret	
; Minuten				
ZEIT_MM:
	STS (adr_ss),NULL	; in SRAM speichern	

	LDS temp,(adr_mm)	; aus SRAM laden	
	inc temp
	cpi temp,60
	brsh ZEIT_HH
	STS (adr_mm),temp	; in SRAM speichern	
	ret	
; Stunden				
ZEIT_HH:
	STS (adr_mm),NULL	; in SRAM speichern	

	LDS temp,(adr_hh)	; aus SRAM laden	
	inc temp
	cpi temp,24
	brsh ZEIT_TAG
	STS (adr_hh),temp	; in SRAM speichern	
	ret	
; TAG					
ZEIT_TAG:
	; ROTDCF			
	cbi (LED_DCF_ROT_PORT),(LED_DCF_ROT_PIN_NR)	 
	STS (adr_hh),NULL	; in SRAM speichern	

; WOCHENTAG				
	LDS temp,(adr_WOCHENTAG)
	inc temp
	cpi temp,7+1
	brlo ZEIT_WOCHENTAG_w
	ldi temp,1
ZEIT_WOCHENTAG_w:
	STS (adr_WOCHENTAG),temp

	LDS temp,(adr_TAG)	; aus SRAM laden	
	inc temp
	STS (adr_TAG),temp	; in SRAM speichern	
	
	rcall ZEIT_TAGE_IM_MONAT
	cp temp,temp1 
	brsh ZEIT_MONAT
	ret
; MONAT					
ZEIT_MONAT:
	STS (adr_TAG),EINS

	LDS temp,(adr_MONAT); aus SRAM laden	
	inc temp
	cpi temp,13 
	brsh ZEIT_JAHR
	STS (adr_MONAT),temp; in SRAM speichern	
	ret
; JAHR					
ZEIT_JAHR:
	STS (adr_MONAT),EINS

	LDS temp,(adr_JAHR)	; aus SRAM laden	
	inc temp
	STS (adr_JAHR),temp	; in SRAM speichern	

ret
; ##############################################################################
; Anzahl der TAGE im Monat	
ZEIT_TAGE_IM_MONAT:
	LDS temp1,(adr_MONAT)
	; vergleich	
	cpi temp1,1			; Januar 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,2			; Februar 28 	
	breq ZEIT_TAGE_IM_MONAT_28
	cpi temp1,3			; Mrz 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,4			; April 30		
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,5			; Mai 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,6			; Juni 30		
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,7			; Juli 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,8			; August 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,8			; September 30	
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,10		; Oktober 31	
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,11		; November 30	
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,12		; Dezember 31	
	breq ZEIT_TAGE_IM_MONAT_31
	ret
ZEIT_TAGE_IM_MONAT_28:
	ldi temp1,28
	ret
ZEIT_TAGE_IM_MONAT_30:
	ldi temp1,30
	ret
ZEIT_TAGE_IM_MONAT_31:
	ldi temp1,31
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZEIT_BETRIEBSSTUNDEN:
	LDS temp,(adr_BS_ss)
	inc temp
	cpi temp,60
	brsh ZEIT_BETRIEBSSTUNDEN_mm
	STS (adr_BS_ss),temp
	ret
ZEIT_BETRIEBSSTUNDEN_mm:
	STS (adr_BS_ss),NULL

	LDS temp,(adr_BS_mm)
	inc temp
	cpi temp,60
	brsh ZEIT_BETRIEBSSTUNDEN_hh
	STS (adr_BS_mm),temp
	ret
ZEIT_BETRIEBSSTUNDEN_hh:
	STS (adr_BS_mm),NULL

	LDS temp,(adr_BS_hh)
	inc temp
	cpi temp,24
	brsh ZEIT_BETRIEBSSTUNDEN_TAG
	STS (adr_BS_hh),temp
	ret
ZEIT_BETRIEBSSTUNDEN_TAG:
	STS (adr_BS_hh),NULL

	LDS ZL,(adr_BS_TAG_L)
	LDS ZH,(adr_BS_TAG_H)
	adiw ZL,1
	STS (adr_BS_TAG_L),ZL
	STS (adr_BS_TAG_H),ZH

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TIMER_PHASE:
	LDS temp,(adr_DCF_PHASE_TIMER)
	tst temp
	breq TIMER_PHASE_w
	dec temp
	STS(adr_DCF_PHASE_TIMER),temp
TIMER_PHASE_w:
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

